/*
  RC Control 6WD Robot and 4DOF
  By: yfrobot-wst
  Date: Mar.2021
  License: CC-BY SA 3.0 - Creative commons share-alike 3.0
  use this code however you'd like, just keep this license and
  attribute. Let me know if you make hugely, awesome, great changes.
*/

#include <Servo.h>
int ch1, ch2, ch3, ch4;

Servo myservo1;
Servo myservo2;
Servo myservo3;

/*********************************************************************
 RC receiver connection port
*********************************************************************/
int RC1 = A0;
int RC2 = A1;
int RC3 = A2;
int RC4 = A3;

/*********************************************************************
 Motor drive port settings, channel A is the left side of the robot, 
 and channel B is the right side of the robot.
*********************************************************************/
#define DIRA    4
#define PWMA    5
#define SLPA    2    //Enable

#define DIRB    7
#define PWMB    6
#define SLPB    8    //Enable

int MAXSPEEDA = 255;   //set Joystick speed
int MAXSPEEDB = 255;   //set Joystick speed
int MIDSPEEDA = MAXSPEEDA;
int MIDSPEEDB = MAXSPEEDB;

/*********************************************************************
 Servo port wiring and default angle position value, the fourth servo 
 is controlled by the sixth channel of the receiver, and the minimum 
 state is the open state.
*********************************************************************/
#define SERVOPIN1    9   //Servo1
#define SERVOPIN2    10  //Servo2
#define SERVOPIN3    11  //Servo3

int SERVOGROUNDIA = 150;    //IA is the initial angle120，110，20
int SERVOMIDIA = 145;
int SERVOUPIA = 20;

int SERVOGROUNDTA1 = 0;     //TA is the Target angle
int SERVOMIDTA1 = 35;
int SERVOUPTA1 = 130;

int SERVOGROUNDTA2 = 0  ;    //TA is the Target angle
int SERVOMIDTA2 = 10;
int SERVOUPTA2 = 25;

void setup() {
  Serial.begin(115200); // Pour a bowl of Serial
  pinMode(RC1, INPUT); // Set our input pins as such
  pinMode(RC2, INPUT);
  pinMode(RC3, INPUT);
  pinMode(RC4, INPUT);

  pinMode(DIRA,  OUTPUT);
  pinMode(PWMA, OUTPUT);
  pinMode(SLPA,  OUTPUT);
  pinMode(DIRB,  OUTPUT);
  pinMode(PWMB, OUTPUT);
  pinMode(SLPB,  OUTPUT);
  digitalWrite(SLPA, HIGH);
  digitalWrite(SLPB, HIGH);

  myservo1.attach(SERVOPIN1);
  myservo2.attach(SERVOPIN2);
  myservo3.attach(SERVOPIN3);

  myservo1.write(SERVOGROUNDIA);
  myservo2.write(SERVOMIDIA);
  myservo3.write(SERVOUPIA);
}

void loop() {
  ch1 = pulseIn(RC1, HIGH, 25000); // Read the pulse width of
  ch2 = pulseIn(RC2, HIGH, 25000); // each channel
  ch3 = pulseIn(RC3, HIGH, 25000);
  ch4 = pulseIn(RC4, HIGH, 25000);

  if (ch2 <= 1020) {
    ch2 = 1020;
  }
  if (ch2 >= 1960) {
    ch2 = 1960;
  }
  if (ch3 <= 1020) {
    ch3 = 1020;
  }
  if (ch3 >= 1960) {
    ch3 = 1960;
  }
  if (ch4 <= 1020) {
    ch4 = 1020;
  }
  if (ch4 >= 1960) {
    ch4 = 1960;
  }
  if (ch2 <= 1400 && ch3 <= 1400 && ch4 <= 1400) {
    digitalWrite(SLPA, LOW);
    digitalWrite(SLPB, LOW);
  } else {
    if (ch2 >= 1020 && ch2 <= 1400) {
      digitalWrite(DIRA, HIGH);
      digitalWrite(DIRB, LOW);
      analogWrite(PWMA, map(ch2, 1020, 1400, MAXSPEEDA, 0));
      analogWrite(PWMB, map(ch2, 1020, 1400, MAXSPEEDB, 0));
    } else if (ch2 >= 1600 && ch2 <= 1960) {
      digitalWrite(DIRA, LOW);
      digitalWrite(DIRB, HIGH);
      analogWrite(PWMA, map(ch2, 1600, 1960, 0, MAXSPEEDA));
      analogWrite(PWMB, map(ch2, 1600, 1960, 0, MAXSPEEDB));  
    } else if (ch4 >= 1020 && ch4 <= 1400) {
      digitalWrite(DIRA, HIGH);
      digitalWrite(DIRB, HIGH);
      analogWrite(PWMA, map(ch4, 1020, 1400, MAXSPEEDA, 0));
      analogWrite(PWMB, map(ch4, 1020, 1400, MAXSPEEDB, 0));
    } else if (ch4 >= 1600 && ch4 <= 1960) {
      digitalWrite(DIRA, LOW);
      digitalWrite(DIRB, LOW);
      analogWrite(PWMA, map(ch4, 1600, 1960, 0, MAXSPEEDA));
      analogWrite(PWMB, map(ch4, 1600, 1960, 0, MAXSPEEDB));
    } else {
      digitalWrite(PWMA, LOW);
      digitalWrite(PWMB, LOW);
    }
    digitalWrite(SLPA, HIGH);
    digitalWrite(SLPB, HIGH);
  }

  if (ch1 > 1800 && ch1 < 2100) {
    if (1020 <= ch3 && ch3 <= 1960) {
      myservo1.write(map(ch3, 1020, 1960, SERVOGROUNDTA2, SERVOGROUNDIA));
      myservo2.write(map(ch3, 1020, 1960, SERVOMIDTA2, SERVOMIDIA));
      myservo3.write(map(ch3, 1020, 1960, SERVOUPTA2, SERVOUPIA));
    }
  }

  if (ch1 > 800 && ch1 < 1300) {
    if (1020 <= ch3 && ch3 <= 1960) {
      myservo1.write(map(ch3, 1020, 1960, SERVOGROUNDTA1, SERVOGROUNDIA));
      myservo2.write(map(ch3, 1020, 1960, SERVOMIDTA1, SERVOMIDIA));
      myservo3.write(map(ch3, 1020, 1960, SERVOUPTA1, SERVOUPIA));
    }
  }
  delay(50);

  Serial.print("Channel 1:"); // Print the value of
  Serial.print(ch1);        // each channel
  Serial.print("   Channel 2:");
  Serial.print(ch2);
  Serial.print("   Channel 3:");
  Serial.print(ch3);
  Serial.print("   Channel 4:");
  Serial.println(ch4);
}
